@@ -66,5 +66,25 @@ mutate.ExprBuilder <- function(.data, ..., .sequential = FALSE, .unquote_names =
         clause <- clauses[[1L]]
     }
 
-    .data$set_select(clause, .chain)
+    .data$set_j(clause, .chain)
+}
+
+#' @rdname mutate-table.express
+#' @export
+#' @importFrom rlang caller_env
+#'
+#' @param .parent_env See [end_expr()]
+#'
+mutate.EagerExprBuilder <- function(.data, ..., .parent_env = rlang::caller_env()) {
+    end_expr.ExprBuilder(mutate.ExprBuilder(.data, ...), .parent_env = .parent_env)
+}
+
+#' @rdname mutate-table.express
+#' @export
+#' @importFrom rlang caller_env
+#'
+mutate.data.table <- function(.data, ...) {
+    eb <- ExprBuilder$new(.data)
+    lazy_ans <- mutate.ExprBuilder(eb, ...)
+    end_expr.ExprBuilder(lazy_ans, .parent_env = rlang::caller_env())
 }
