@@ -30,7 +30,6 @@ order_by <- function(.data, ...) { UseMethod("order_by") }
 #' data("mtcars")
 #'
 #' data.table::as.data.table(mtcars) %>%
-#'     start_expr %>%
 #'     order_by(-cyl, gear)
 #'
 order_by.ExprBuilder <- function(.data, ..., .collapse,
@@ -39,3 +38,13 @@ order_by.ExprBuilder <- function(.data, ..., .collapse,
 {
     where.ExprBuilder(.data, order(!!!parse_dots(.parse, ...)), .parse = FALSE, .chain = .chain)
 }
+
+#' @rdname order_by-table.express
+#' @export
+#' @importFrom rlang caller_env
+#'
+order_by.data.table <- function(.data, ...) {
+    eb <- ExprBuilder$new(.data)
+    lazy_ans <- order_by.ExprBuilder(eb, ...)
+    end_expr.ExprBuilder(lazy_ans, .parent_env = rlang::caller_env())
+}
